using System;
using System.Transactions;

namespace TheRM
{
  public class VolatileRM : IEnlistmentNotification
  {
    private string _whoAmI = string.Empty;
    public VolatileRM(string whoAmI)
    {
      this._whoAmI = whoAmI;
    }

    private int _memberValue = 0;
    private int _oldMemberValue = 0;
    public int MemberValue
    {
      get
      {
        return this._memberValue;
      }
      set
      {
        Transaction tran = Transaction.Current;
        if (tran != null)
        {
          Console.WriteLine(
           this._whoAmI + ": MemberValue setter - EnlistVolatile");
          tran.EnlistVolatile(this, EnlistmentOptions.None);
        }
        this._oldMemberValue = this._memberValue;
        this._memberValue = value;
      }
    }

    #region IEnlistmentNotification Members

    public void Commit(Enlistment enlistment)
    {
      Console.WriteLine(this._whoAmI + ": Commit");

      // Clear out _oldMemberValue 
      this._oldMemberValue = 0;
      enlistment.Done();
    }

    public void InDoubt(Enlistment enlistment)
    {
      Console.WriteLine(this._whoAmI + ": InDoubt");
      enlistment.Done();
    }

    public void Prepare(PreparingEnlistment preparingEnlistment)
    {
      Console.WriteLine(this._whoAmI + ": Prepare");
      preparingEnlistment.Prepared();
      //preparingEnlistment.ForceRollback();
    }

    public void Rollback(Enlistment enlistment)
    {
      Console.WriteLine(this._whoAmI + ": Rollback");

      // Restore previous state 
      _memberValue = _oldMemberValue;
      _oldMemberValue = 0;
      enlistment.Done();
    }

    #endregion
  }
}