-- Create a DMK and Certificate in master
USE master
GO

-- Create database master key (DMK)
CREATE MASTER KEY
 ENCRYPTION BY PASSWORD = 'Hrd2GessP@ssw0rd!'

-- Create TDE certificate
CREATE CERTIFICATE MyEncryptionCert
 WITH SUBJECT = 'My Encryption Certificate'

-- Verify that the certificate has been created and is protected by the DMK
SELECT name, pvt_key_encryption_type_desc FROM sys.certificates
 WHERE name = 'MyEncryptionCert'


-- Create a new database
IF EXISTS(SELECT name FROM sys.databases WHERE name = 'MyDB')
 DROP DATABASE MyDB 
GO
CREATE DATABASE MyDB
GO
USE MyDB
GO

-- Store some unencrypted sensitive data
CREATE TABLE CreditCardInfo
(CardType varchar(50),
 CardNumber varchar(20)) 
GO 

INSERT INTO CreditCardInfo VALUES
 ('MasterCard', '1234-1234-1234-1234'),
 ('American Express', '9876-987653-98765')

-- Create DEK
CREATE DATABASE ENCRYPTION KEY
 WITH ALGORITHM = AES_128
 ENCRYPTION BY SERVER CERTIFICATE MyEncryptionCert


-- Enable TDE on the database
ALTER DATABASE MyDB SET ENCRYPTION ON

-- Verify TDE is enabled on the database
SELECT name, is_encrypted
 FROM sys.databases
 WHERE is_encrypted = 1


-- Monitor encryption progress
SELECT
   sys.databases.name,
   sys.dm_database_encryption_keys.encryption_state,
   sys.dm_database_encryption_keys.percent_complete,
   sys.dm_database_encryption_keys.key_algorithm,
   sys.dm_database_encryption_keys.key_length
 FROM
   sys.dm_database_encryption_keys INNER JOIN sys.databases
   ON sys.dm_database_encryption_keys.database_id = sys.databases.database_id
 WHERE
   sys.databases.name = 'MyDB'


-- Backup the certificate and its private key (protecting the private key backup with a password)
USE master
GO

BACKUP CERTIFICATE MyEncryptionCert TO FILE='C:\Demo\Backups\MyEncryptionCert.certbak'
 WITH PRIVATE KEY (
  FILE='C:\Demo\Backups\MyEncryptionCert.pkbak', 
  ENCRYPTION BY PASSWORD='Pr!vK3yP@ssw0rd')


-- Simulate disaster requiring recovery to a new server
USE master
GO

-- Lose the database
DROP DATABASE MyDB
GO

-- Lose the TDE certificate
DROP CERTIFICATE MyEncryptionCert

-- Lose the DMK and create a new one
DROP MASTER KEY
CREATE MASTER KEY ENCRYPTION BY PASSWORD = 'An0thrHrd2GessP@ssw0rd!'


-- Restore the TDE certificate
CREATE CERTIFICATE MyEncryptionCert
 FROM FILE='C:\Demo\Backups\MyEncryptionCert.certbak'
 WITH PRIVATE KEY(
  FILE='C:\Demo\Backups\MyEncryptionCert.pkbak',
  DECRYPTION BY PASSWORD='Pr!vK3yP@ssw0rd')







-- Cleanup
USE MyDB
GO
DROP TABLE CreditCardInfo
USE master
GO
DROP DATABASE MyDB
DROP CERTIFICATE MyEncryptionCert
DROP MASTER KEY
