-- Set FILESTREAM access level
EXEC sp_configure filestream_access_level, 2
RECONFIGURE

-- Creating a FILESTREAM-enabled catabase with FILEGROUPCONTAINS FILESTREAM
CREATE DATABASE PhotoLibrary
 ON PRIMARY
  (NAME = PhotoLibrary_data, 
   FILENAME = 'C:\Demo\PhotoLibrary\PhotoLibrary_data.mdf'),
 FILEGROUP FileStreamGroup1 CONTAINS FILESTREAM
  (NAME = PhotoLibrary_group2, 
   FILENAME = 'C:\Demo\PhotoLibrary\Photos')
 LOG ON 
  (NAME = PhotoLibrary_log,
   FILENAME = 'C:\Demo\PhotoLibrary\PhotoLibrary_log.ldf')
GO

-- Creating a FILESTREAM-enabled table
USE PhotoLibrary
GO

CREATE TABLE PhotoAlbum(
 PhotoId int PRIMARY KEY,
 RowId uniqueidentifier ROWGUIDCOL NOT NULL UNIQUE DEFAULT NEWSEQUENTIALID(),
 Description varchar(max),
 Photo varbinary(max) FILESTREAM DEFAULT(0x))

-- Insert a text BLOB
INSERT INTO PhotoAlbum(PhotoId, Description, Photo)
 VALUES(1, 'Text pic', CAST('BLOB' As varbinary(max)))

-- Select it back
SELECT *, CAST(Photo AS varchar) AS PhotoText FROM PhotoAlbum
GO

-- Insert an image BLOB
INSERT INTO PhotoAlbum(PhotoId, Description, Photo)
 VALUES(2, 'Document icon', 0x4749463839610C000E00B30000FFFFFFC6DEC6C0C0C0000080000000D3121200000000000000000000000000000000000000000000000000000000000021F90401000002002C000000000C000E0000042C90C8398525206B202F1820C80584806D1975A29AF48530870D2CEDC2B1CBB6332EDE35D9CB27DCA554484204003B)

/*
-- Or, if the BLOB exists in an external file:
INSERT INTO PhotoAlbum(PhotoId, Description, Photo)
 VALUES(2, 'Document icon',
  (SELECT BulkColumn FROM OPENROWSET(BULK '\\public\shared\doc.ico', SINGLE_BLOB) AS x))
*/
